#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <libharkio3.h>






void help(void)
{
    printf("                                                                    \n");
    printf("                                                                    \n");
    printf("[NAME:]                                                             \n");
    printf("  harktoolcli-genconfig                                             \n");
    printf("                                                                    \n");
    printf("[Description:]                                                      \n");
    printf("  The command creates a xml file that contians a config tag         \n");
    printf("                                                                    \n");
    printf("[Usage :]                                                           \n");
    printf("  harktoolcli-genconfig output_file_name [options]                  \n");
    printf("                                                                    \n");
    printf("[options]                                                           \n");
    printf("  -s: Number of TSPs in the recorded wav (-s 16)    [integer]       \n");
    printf("  -o: Offset of the TSP                  (-o 0)     [integer]       \n");
    printf("  -p: Range for peak search              (-p 0,32)  [integer,integer] \n");
    printf("  -n: NFFT of convolution                (-n 1024)  [integer]       \n");
    printf("  -r: SamplingRate                       (-r 16000) [integer]       \n");
    printf("  -m: SignalMax                          (-m 32768) [integer]       \n");
    printf("  -l: TSPlength                          (-l 16384) [integer]       \n");
    printf("  -t: Path to the original TSP                      [string]        \n");
    printf("                                                                    \n");
    printf("                                                                    \n");
}









int main(int argc, char **argv)
{

    harkio_XML *xml;
    FILE *fp;
    int option;
    char *buf;



    harkio_Config *cfg = harkio_Config_new();


    if (argc <= 1) {
        harkio_Log_printf(harkio_Log_Error , "argument error!!");
        help();
        exit(EXIT_FAILURE);
    }

    // help
    if (strcmp(argv[1] , "--help") == 0) {
        help();
        exit(EXIT_SUCCESS);
    }

    if ((fp = fopen(argv[1], "w")) == NULL) {
        harkio_Log_FileOpenFailed(argv[1]);
        exit(EXIT_FAILURE);
    }


    // set default values
    harkio_Config_setComment(cfg,     "comment");
    harkio_Config_setSynchronousAverage(cfg, 16); //Number of TSPs in the recorded wav
    harkio_Config_setTSPoffset(cfg,           0); //Offset of the TSP
    harkio_Config_setPeakSearchFrom(cfg,      0); //Range for peak search
    harkio_Config_setPeakSearchTo(cfg,       32);
    harkio_Config_setNfft(cfg,             1024); //NFFT of convolution
    harkio_Config_setTSPpath(cfg,            ""); //Path to the original TSP
    harkio_Config_setSamplingRate(cfg,    16000);
    harkio_Config_setSignalMax(cfg,       32768);
    harkio_Config_setTSPlength(cfg,       16384);



    // parse argv using getopt
    while ((option = getopt(argc, argv, "s:o:p:n:t:r:m:l:")) != -1) {
        switch (option) {
        case 's':
            harkio_Config_setSynchronousAverage(cfg, atoi(optarg));
            break;

        case 't':
            harkio_Config_setTSPpath(cfg, (const char*) optarg);
            break;

        case 'o':
            harkio_Config_setTSPoffset(cfg, atoi(optarg));
            break;

        case 'p':
            buf = strtok(optarg, ",");
            harkio_Config_setPeakSearchFrom(cfg, atoi(buf));
            buf = strtok(NULL, ",");
            harkio_Config_setPeakSearchTo(cfg, atoi(buf));
            break;

        case 'n':
            if (harkio_Config_setNfft(cfg, atoi(optarg)) == EXIT_FAILURE) {
                harkio_Log_printf(harkio_Log_Error, "genConfig Error n");
                exit(1);
            }
            break;

        case 'r':
            if (harkio_Config_setSamplingRate(cfg, atoi(optarg)) == EXIT_FAILURE) {
                harkio_Log_printf(harkio_Log_Error, "genConfig Error r");
                exit(1);
            }
            break;

        case 'm':
            if (harkio_Config_setSignalMax(cfg, atoi(optarg)) == EXIT_FAILURE) {
                harkio_Log_printf(harkio_Log_Error, "genConfig Error m");
                exit(1);
            }
            break;

        case 'l':
            if (harkio_Config_setTSPlength(cfg, atoi(optarg)) == EXIT_FAILURE) {
                harkio_Log_printf(harkio_Log_Error, "genConfig Error l");
                exit(1);
            }
            break;

        case '?':
            harkio_Config_delete(&cfg);
            help();
            exit(EXIT_FAILURE);

        }
    }


    // debug print
    harkio_Config_print(cfg);

    // make new XML object and write
    xml = harkio_XML_newFromData(NULL, NULL, cfg);

    harkio_XML_write(xml, fp);

    harkio_XML_delete(&xml);

    fclose(fp);


    exit(EXIT_SUCCESS);

}
